using System;
using System.Collections.Generic;
using System.Linq;
using System.Net.Http;
using System.Threading.Tasks;
using FuzzySharp;
using FuzzySharp.Extractor;
using Props.Shop.Framework;

namespace Props.Shop.Adafruit.Api
{
    public class ProductListingManager
    {
        private double minutesPerRequest;
        private Dictionary<string, List<ProductListing>> listings = new Dictionary<string, List<ProductListing>>();
        private bool requested = false;
        public DateTime TimeOfLastRequest { get; private set; }
        public bool RequestReady => !requested || DateTime.Now - TimeOfLastRequest > TimeSpan.FromMinutes(minutesPerRequest);

        public ProductListingManager(int requestsPerMinute = 5)
        {
            this.minutesPerRequest = 1 / requestsPerMinute;
        }

        public async Task RefreshListings(HttpClient http)
        {
            requested = true;
            TimeOfLastRequest = DateTime.Now;
            HttpResponseMessage response = await http.GetAsync("/products");
            SetListingsData(await response.Content.ReadAsStringAsync());
        }

        public void SetListingsData(string data)
        {
            ListingsParser listingsParser = new ListingsParser(data);
            foreach (ProductListing listing in listingsParser.ProductListings)
            {
                List<ProductListing> similar = listings.GetValueOrDefault(listing.Name, new List<ProductListing>());
                similar.Add(listing);
                listings[listing.Name] = similar;
            }
        }

        public async IAsyncEnumerable<ProductListing> Search(string query, float similarity, HttpClient httpClient = null)
        {
            if (RequestReady && httpClient != null) await RefreshListings(httpClient);
            IEnumerable<ExtractedResult<string>> resultNames = Process.ExtractAll(query, listings.Keys, cutoff: (int)similarity * 100);
            foreach (ExtractedResult<string> resultName in resultNames)
            {
                foreach (ProductListing product in listings[resultName.Value])
                {
                    yield return product;
                }
            }
        }

    }
}